@@ -1,6 +1,7 @@
 /* actions.c
  *
  * Copyright 2002 Lutz Mueller <lutz@users.sourceforge.net>
+ * Copyright 2016 Marcus Meissner <marcus@jet.franken.de>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,9 +22,14 @@
 #define _XOPEN_SOURCE	/* strptime proto, but this hides other prototypes */
 #define _GNU_SOURCE	/* get all the other prototypes */
 #define __EXTENSIONS__	/* for solaris to get back strdup and strcasecmp */
+#define _DARWIN_C_SOURCE
 
 #include "config.h"
+#if defined (HAVE_SIGNAL_H)
+#include <signal.h>
+#endif
 
+#include <libgen.h>
 #include <string.h>
 #include <stdio.h>
 #ifdef HAVE_FCNTL_H
@@ -906,7 +912,7 @@ print_version_action (GPParams __unused__ *p)
 #endif
 	printf (_("gphoto2 %s\n"
 		  "\n"
-		  "Copyright (c) 2000-%d Lutz Mueller and others\n"
+		  "Copyright (c) 2000-%d Marcus Meissner and others\n"
 		  "%s"
 		  "\n"
 		  "gphoto2 comes with NO WARRANTY, to the extent permitted by law. You may\n"
@@ -915,7 +921,7 @@ print_version_action (GPParams __unused__ *p)
 		  "\n"
 		  "This version of gphoto2 is using the following software versions and options:\n"),
 		VERSION,
-		2016, /* year of release! */
+		2021, /* year of release! */
 		port_message
 		);
 
@@ -947,21 +953,26 @@ _action_camera_capture_preview (GPParams *p, int viewasciiart)
 {
 	CameraFile *file;
 	int	r, fd;
-        char    tmpname[20], *tmpfilename;
-
-	strcpy (tmpname, "tmpfileXXXXXX");
-	fd = mkstemp(tmpname);
-	if (fd == -1) {
-		CR (gp_file_new (&file));
-		tmpfilename = NULL;
+	char tmpname[20], *tmpfilename = NULL;
+	
+	if (p->flags & FLAGS_STDOUT) {
+		fd = dup(fileno(stdout));
+		CR (gp_file_new_from_fd (&file, fd));
 	} else {
-		r = gp_file_new_from_fd (&file, fd);
-		if (r < GP_OK) {
-			close (fd);
-			unlink (tmpname);
-			return r;
+		strcpy (tmpname, "tmpfileXXXXXX");
+		fd = mkstemp(tmpname);
+		if (fd == -1) {
+			CR (gp_file_new (&file));
+			tmpfilename = NULL;
+		} else {
+			r = gp_file_new_from_fd (&file, fd);
+			if (r < GP_OK) {
+				close (fd);
+				unlink (tmpname);
+				return r;
+			}
+			tmpfilename = tmpname;
 		}
-		tmpfilename = tmpname;
 	}
 
 #ifdef HAVE_AA
@@ -970,21 +981,24 @@ _action_camera_capture_preview (GPParams *p, int viewasciiart)
 	else
 #endif
 		r = gp_camera_capture_preview (p->camera, file, p->context);
-
+	fflush(stdout);
 	if (r < 0) {
-		gp_file_unref (file);
+		if(!(p->flags & FLAGS_STDOUT))
+			gp_file_unref (file);
 		unlink (tmpname);
-		return (r);
+		return r;
 	}
 
 	/* name it file_%filename if --filename is set, otherwise capture_preview */
-	r = save_camera_file_to_file (NULL, "capture_preview", p->filename?GP_FILE_TYPE_PREVIEW:GP_FILE_TYPE_NORMAL, file, tmpfilename);
-	gp_file_unref (file);
-	if (r < 0) {
-		unlink (tmpname);
-		return (r);
+	if(!(p->flags & FLAGS_STDOUT)) {
+		r = save_camera_file_to_file (NULL, "capture_preview", p->filename?GP_FILE_TYPE_PREVIEW:GP_FILE_TYPE_NORMAL, file, tmpfilename);
+		gp_file_unref (file);
+		if (r < 0) {
+			unlink (tmpname);
+			return (r);
+		}
 	}
-	return (GP_OK);
+	return GP_OK;
 }
 
 int
@@ -1003,14 +1017,14 @@ int
 action_camera_capture_movie (GPParams *p, const char *arg)
 {
 	CameraFile	*file;
-	int		r;
+	int		r, tries;
 	int		fd;
 	time_t		st;
 	enum moviemode	mm;
 	int		frames,captured_frames=0;
 	char		*xname;
 	struct timeval	starttime;
-
+	
 	if (p->flags & FLAGS_STDOUT) {
 		fd = dup(fileno(stdout));
 		xname = "stdout";
@@ -1039,13 +1053,20 @@ action_camera_capture_movie (GPParams *p, const char *arg)
 	}
 	CR (gp_file_new_from_fd (&file, fd));
 	gettimeofday (&starttime, NULL);
+	tries = 0;
 	while (1) {
 		const char *mime;
 		r = gp_camera_capture_preview (p->camera, file, p->context);
 		if (r < 0) {
+			if (r == GP_ERROR_CAMERA_BUSY) {
+				/* allow 20 busy tries */
+				if (tries++ < 20)
+					continue; /* just continue */
+			}
 			cli_error_print(_("Movie capture error... Exiting."));
 			break;
 		}
+		tries = 0;
 		gp_file_get_mime_type (file, &mime);
                 if (strcmp (mime, GP_MIME_JPEG)) {
 			cli_error_print(_("Movie capture error... Unhandled MIME type '%s'."), mime);
@@ -1093,6 +1114,8 @@ action_camera_wait_event (GPParams *p, enum download_type downloadtype, const ch
 	struct timeval	xtime;
 	int events, frames;
 
+        end_next = 0;
+
 	gettimeofday (&xtime, NULL);
 	memset(&last,0,sizeof(last));
 
@@ -1122,7 +1145,7 @@ action_camera_wait_event (GPParams *p, enum download_type downloadtype, const ch
 		} else {
 			wp.type = WAIT_STRING;
 			wp.u.str = arg;
-			printf ( _("Waiting for %s event from camera. Press Ctrl-C to abort.\n"), wp.u.str);
+			printf ( _("Waiting for '%s' event from camera. Press Ctrl-C to abort.\n"), wp.u.str);
 		}
 	}
 
@@ -1134,6 +1157,12 @@ action_camera_wait_event (GPParams *p, enum download_type downloadtype, const ch
 
 		if (glob_cancel) break;
 
+		if (end_next) {
+			printf(_("SIGUSR2 signal received, stopping wait!\n"));
+			end_next = 0;
+			break;
+		}
+
 		exitloop = 0;
 		switch (wp.type) {
 		case WAIT_STRING:
@@ -1176,6 +1205,10 @@ action_camera_wait_event (GPParams *p, enum download_type downloadtype, const ch
 			break;
 		case GP_EVENT_TIMEOUT:
 			/*printf("TIMEOUT\n");*/
+			if ((wp.type == WAIT_STRING) && strstr("TIMEOUT",wp.u.str)) {
+				printf(_("event found, stopping wait!\n"));
+				return GP_OK;
+			}
 			break;
 		case GP_EVENT_CAPTURE_COMPLETE:
 			printf("CAPTURECOMPLETE\n");
@@ -1246,6 +1279,14 @@ action_camera_wait_event (GPParams *p, enum download_type downloadtype, const ch
 				return GP_OK;
 			}
 			break;
+		case GP_EVENT_FILE_CHANGED:
+			fn = (CameraFilePath*)data;
+			printf("FILECHANGED %s %s\n",fn->name, fn->folder);
+			if ((wp.type == WAIT_STRING) && strstr("FILECHANGED",wp.u.str)) {
+				printf(_("event found, stopping wait!\n"));
+				return GP_OK;
+			}
+			break;
 		}
 		free (data);
 	}
@@ -1338,6 +1379,9 @@ print_storage_info (GPParams *p)
 		if (sinfos[i].fields & GP_STORAGEINFO_FREESPACEIMAGES)
 			printf ("freeimages=%lu\n", (unsigned long)sinfos[i].freeimages);
 	}
+	if (sinfos)
+		free(sinfos);
+
 	return GP_OK;
 }
 
@@ -1615,7 +1659,7 @@ static int
 print_widget (GPParams *p, const char *name, CameraWidget *widget) {
 	const char *label;
 	CameraWidgetType	type;
-	int ret;
+	int ret, readonly;
 
 	ret = gp_widget_get_type (widget, &type);
 	if (ret != GP_OK)
@@ -1623,8 +1667,13 @@ print_widget (GPParams *p, const char *name, CameraWidget *widget) {
 	ret = gp_widget_get_label (widget, &label);
 	if (ret != GP_OK)
 		return ret;
+		
+	ret = gp_widget_get_readonly (widget, &readonly);
+	if (ret != GP_OK)
+		return ret;
 
 	printf ("Label: %s\n", label); /* "Label:" is not i18ned, the "label" variable is */
+	printf ("Readonly: %d\n", readonly);
 	switch (type) {
 	case GP_WIDGET_TEXT: {		/* char *		*/
 		char *txt;
@@ -1717,6 +1766,8 @@ print_widget (GPParams *p, const char *name, CameraWidget *widget) {
 	case GP_WIDGET_BUTTON:
 		break;
 	}
+	
+	printf ("END\n");
 	return GP_OK;
 }
 
@@ -2062,8 +2113,10 @@ set_config_value_action (GPParams *p, const char *name, const char *value) {
 		int	t = -1;
 		struct tm xtm;
 
+		if (!strcasecmp (value, "now")  || !strcasecmp (value, _("now")))
+			t = time(NULL);
 #ifdef HAVE_STRPTIME
-		if (strptime (value, "%c", &xtm) || strptime (value, "%Ec", &xtm))
+		else if (strptime (value, "%c", &xtm) || strptime (value, "%Ec", &xtm))
 			t = mktime (&xtm);
 #endif
 		if (t == -1) {
