@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #include "config.h"
@@ -317,11 +317,12 @@ int
 for_each_file_in_range (GPParams *p, FileAction action,
 			const char *range)
 {
-	char	index[MAX_IMAGE_NUMBER];
+	char	*index;
 	int 	i, max = 0, r;
-	char ffolder[MAX_FOLDER_LEN], ffile[MAX_FILE_LEN];
+	char 	ffolder[MAX_FOLDER_LEN], ffile[MAX_FILE_LEN];
 
-	memset(index, 0, MAX_IMAGE_NUMBER);
+	index = calloc(MAX_IMAGE_NUMBER,1);
+	if (!index) return GP_ERROR_NO_MEMORY;
 
 	CR (parse_range (range, index, p->context));
 
