@@ -14,10 +14,12 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
+#define _XOPEN_SOURCE
+#define _GNU_SOURCE
 #include "config.h"
 #include "gphoto2-cmd-config.h"
 #include "i18n.h"
@@ -27,7 +29,13 @@
 
 #include <gphoto2/gphoto2-widget.h>
 
+#ifdef HAVE_CDK_CDK_H
 #include <cdk/cdk.h>
+#else
+#ifdef HAVE_CDK_H
+#include <cdk.h>
+#endif
+#endif
 
 #ifndef MAX
 # define MAX(a, b) ((a) > (b) ? (a) : (b))
@@ -51,8 +59,8 @@ static int
 set_config (CmdConfig *cmd_config)
 {
 	int result, selection;
-	char *msg[10];
-	char *buttons[] = {N_("</B/24>Continue"), N_("</B16>Cancel")};
+	const char *msg[10];
+	const char *buttons[] = {N_("</B/24>Continue"), N_("</B16>Cancel")};
 	CDKDIALOG *question = NULL;
 
 	result = gp_camera_set_config (cmd_config->camera, cmd_config->window, 
@@ -359,7 +367,8 @@ show_toggle (CmdConfig *cmd_config, CameraWidget *toggle)
 	CDKITEMLIST *list = NULL;
 	int value, selection;
 	const char *label;
-	char title[1024], *info[] = {N_("Yes"), N_("No")};
+	char title[1024];
+	const char *info[] = {N_("Yes"), N_("No")};
 
 	CHECK (gp_widget_get_value (toggle, &value));
 	CHECK (gp_widget_get_label (toggle, &label));
