@@ -14,10 +14,12 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
+#define _DARWIN_C_SOURCE
+
 #include "config.h"
 #include "actions.h"
 #include "globals.h"
@@ -95,12 +97,15 @@ static int shell_set_config    (Camera *, const char *);
 static int shell_set_config_index    (Camera *, const char *);
 static int shell_set_config_value    (Camera *, const char *);
 static int shell_capture_image (Camera *, const char *);
+static int shell_trigger_capture (Camera *, const char *);
 static int shell_capture_tethered (Camera *, const char *);
 static int shell_capture_image_and_download (Camera *, const char *);
 static int shell_capture_preview (Camera *, const char *);
 static int shell_mkdir         (Camera *, const char *);
 static int shell_rmdir         (Camera *, const char *);
 static int shell_wait_event    (Camera *, const char *);
+static int shell_summary       (Camera *, const char *);
+static int shell_storage_info  (Camera *, const char *);
 
 #define MAX_FOLDER_LEN 1024
 #define MAX_FILE_LEN 1024
@@ -121,7 +126,7 @@ static const struct _ShellFunctionTable {
 	{"cd", shell_cd, N_("Change to a directory on the camera"),
 	 N_("directory"), 1},
 	{"lcd", shell_lcd, N_("Change to a directory on the local drive"),
-	 N_("directory"), 1},
+	 N_("directory"), 0},
 	{"exit", shell_exit, N_("Exit the gPhoto shell"), NULL, 0},
 	{"get", shell_get, N_("Download a file"), N_("[directory/]filename"), 1},
 	{"put", shell_put, N_("Upload a file"), N_("[directory/]filename"), 1},
@@ -129,24 +134,27 @@ static const struct _ShellFunctionTable {
 	 N_("[directory/]filename"), 1},
 	{"get-raw", shell_get_raw, N_("Download raw data"),
 	 N_("[directory/]filename"), 1},
-	{"show-info", shell_show_info, N_("Show info"),
+	{"show-info", shell_show_info, N_("Show image information, like width, height, and capture time"),
 	 N_("[directory/]filename"), 1},
 	{"delete", shell_del, N_("Delete"), N_("[directory/]filename"), 1},
 	{"mkdir", shell_mkdir, N_("Create Directory"), N_("directory"), 1},
 	{"rmdir", shell_rmdir, N_("Remove Directory"), N_("directory"), 1},
 #ifdef HAVE_LIBEXIF
-	{"show-exif", shell_show_exif, N_("Show EXIF information"),
+	{"show-exif", shell_show_exif, N_("Show EXIF information of JPEG images"),
 	 N_("[directory/]filename"), 1},
 #endif
 	{"help", shell_help, N_("Displays command usage"),
 	 N_("[command]"), 0},
 	{"ls", shell_ls, N_("List the contents of the current directory"),
 	 N_("[directory/]"), 0},
+	{"summary", shell_summary, N_("Show camera summary"), NULL, 0},
+	{"storage-info", shell_storage_info, N_("Show storage information"), NULL, 0},
 	{"list-config", shell_list_config, N_("List configuration variables"), NULL, 0},
 	{"get-config", shell_get_config, N_("Get configuration variable"), N_("name"), 1},
 	{"set-config", shell_set_config, N_("Set configuration variable"), N_("name=value"), 1},
 	{"set-config-index", shell_set_config_index, N_("Set configuration variable index"), N_("name=valueindex"), 1},
 	{"set-config-value", shell_set_config_value, N_("Set configuration variable"), N_("name=value"), 1},
+	{"trigger-capture", shell_trigger_capture, N_("Triggers the capture of a single image"), NULL, 0},
 	{"capture-image", shell_capture_image, N_("Capture a single image"), NULL, 0},
 	{"capture-image-and-download", shell_capture_image_and_download, N_("Capture a single image and download it"), NULL, 0},
 	{"capture-preview", shell_capture_preview, N_("Capture a preview image"), NULL, 0},
@@ -186,7 +194,7 @@ shell_arg_count (const char *args)
 static char *
 shell_read_line (void)
 {
-	char prompt[70], buf[1024], *line;
+	char prompt[1024+70], buf[1024+50], *line;
 #ifndef HAVE_RL
 	char *tmp;
 #endif
@@ -613,7 +621,7 @@ shell_cd (Camera __unused__ *camera, const char *arg)
 	int arg_count = shell_arg_count (arg);
 
 	if (!arg_count)
-		return (GP_OK);
+		return GP_OK;
 
 	/* shell_arg(arg, 0, arg_dir); */
 
@@ -822,6 +830,18 @@ shell_list_config (Camera __unused__ *camera, const char __unused__ *args) {
 	return (GP_OK);
 }
 
+static int
+shell_summary (Camera __unused__ *camera, const char __unused__ *args) {
+	CHECK (action_camera_summary (p));
+	return GP_OK;
+}
+
+static int
+shell_storage_info (Camera __unused__ *camera, const char __unused__ *args) {
+	CHECK (print_storage_info (p));
+	return GP_OK;
+}
+
 static int
 shell_get_config (Camera __unused__ *camera, const char *args) {
 	char arg[1024];
@@ -898,6 +918,11 @@ shell_set_config_index (Camera __unused__ *camera, const char *args) {
 }
 
 
+static int
+shell_trigger_capture (Camera __unused__ *camera, const char __unused__ *args) {
+	return trigger_capture ();
+}
+
 static int
 shell_capture_image (Camera __unused__ *camera, const char __unused__ *args) {
 	return capture_generic (GP_CAPTURE_IMAGE, NULL, 0);
@@ -916,12 +941,18 @@ shell_capture_preview (Camera __unused__ *camera, const char __unused__ *args) {
 
 static int
 shell_wait_event (Camera *camera, const char *args) {
-	return action_camera_wait_event (p, DT_NO_DOWNLOAD, args);
+	char argument[1024];
+
+	shell_arg (args, 0, argument);
+	return action_camera_wait_event (p, DT_NO_DOWNLOAD, argument);
 }
 
 static int
 shell_capture_tethered (Camera *camera, const char *args) {
-	return action_camera_wait_event (p, DT_DOWNLOAD, args);
+	char argument[1024];
+
+	shell_arg (args, 0, argument);
+	return action_camera_wait_event (p, DT_DOWNLOAD, argument);
 }
 
 
